@@ -325,8 +325,11 @@ class HighScores (GameSelector):
         
     def get_puzzle (self):
         mod,itr = self.tv.get_selection().get_selected()
+        if not itr:
+            return None
         while mod.iter_parent(itr):
             itr = mod.iter_parent(itr)
+            if not itr: return None
         return mod.get_value(itr,5)
 
 class GamePrinter (NewGameSelector):
@@ -487,12 +490,10 @@ class GamePrinter (NewGameSelector):
             # First priority, we last nudged higher and move to the lower end
             if self.last_adjusted == 1 and lower:
                 while len(choices)<nsudokus and lower:
-                    print 'Nudging lower'
                     choices.append(lower.pop())
             # Otherwise, try moving higher...
             if self.last_adjusted == 0 and higher:
                 while len(choices)<nsudokus and higher:
-                    print 'Nudging higher'
                     choices.append(higher.pop())
             # If we still don't have enough, nudge as necessary in both directions...
             just_tried = 0
