@@ -1,5 +1,5 @@
 import gtk, gobject, gtk.glade
-import gnome, gnome.ui
+import gnome, gnome.ui, pango
 import os, os.path
 from gtk_goodies import gconf_wrapper, Undo, dialog_extras, image_extras
 import gsudoku, sudoku, saver, sudoku_maker, printing, sudoku_generator_gui
@@ -9,6 +9,7 @@ from gettext import gettext as _
 from gettext import ngettext
 from defaults import *
 from timer import ActiveTimer
+from simple_debug import simple_debug
 
 
 icon_factory = gtk.IconFactory()
@@ -52,15 +53,15 @@ class UI (gconf_wrapper.GConfWrapper):
       <menu action="View">
         <menuitem action="ZoomIn"/>
         <menuitem action="ZoomOut"/>
+        <menuitem action="FullScreen"/>
         <separator/>
         <menuitem action="ZoomOnResize"/>
         <separator/>
         <menuitem action="ToggleToolbar"/>
-        <separator/>
-        <menuitem action="HighScores"/>
       </menu>
-      <menu action="Hints">
+      <menu action="Game">
          <menuitem action="ShowPossible"/>
+          <menuitem action="AutofillCurrentSquare"/>
           <menuitem action="Autofill"/>
           <separator/>
           <menuitem action="AlwaysShowPossible"/>
@@ -69,6 +70,8 @@ class UI (gconf_wrapper.GConfWrapper):
         <menuitem action="Tracker"/>
         <separator/>
         <menuitem action="PuzzleInfo"/>
+        <separator/>
+        <menuitem action="HighScores"/>
        </menu>
        <menu action="Help">
          <menuitem action="About"/>
@@ -88,7 +91,7 @@ class UI (gconf_wrapper.GConfWrapper):
       <separator/>
       <toolitem action="ShowPossible"/>
       <!--<toolitem action="AlwaysShowPossible"/>-->
-      <toolitem action="Autofill"/>
+      <toolitem action="AutofillCurrentSquare"/>
       <toolitem action="Tracker"/>
       <separator/>
       <toolitem action="ZoomIn"/>
@@ -102,11 +105,14 @@ class UI (gconf_wrapper.GConfWrapper):
                      'always_show_hints':0,
                      'player':os.environ.get('USERNAME',''),
                      'difficulty':0.0,
+                     'minimum_font_size':pango.SCALE * 7, # minimum font-size
                      'minimum_number_of_new_puzzles':MIN_NEW_PUZZLES,
-                     }
+                     }    
 
+    @simple_debug
     def __init__ (self):
         self.w = gtk.Window()
+        self.w.set_default_size(800,600)
         self.timer = ActiveTimer(self.w)
         self.won = False
         gconf_wrapper.GConfWrapper.__init__(self,
@@ -118,7 +124,7 @@ class UI (gconf_wrapper.GConfWrapper):
         self.cleared = [] # used for Undo memory
 
         gnome.program_init(APPNAME,VERSION)
-        self.w.connect('destroy',self.quit_cb)
+        self.w.connect('delete-event',self.quit_cb)
         self.vb = gtk.VBox()
         self.uimanager = gtk.UIManager()
         self.gsd = gsudoku.SudokuGridDisplay()
@@ -146,13 +152,15 @@ class UI (gconf_wrapper.GConfWrapper):
             ('Open',gtk.STOCK_OPEN,_('_Resume old game'),
              '<Control>r','Resume a previous saved game.',
              self.open_game),
-            ('Hints',None,'H_ints'),
+            ('Game',None,'_Game'),
             ('View',None,'_View'),
             ('ShowPossible',gtk.STOCK_HELP,'_Hint',
              '<Control>i',
              'Show which numbers could go in the current square.',
              self.show_hint_cb),
-            ('Autofill',gtk.STOCK_APPLY,'_Fill','<Control>a','Automatically fill in all squares for which there is only one valid value.',
+            ('AutofillCurrentSquare',gtk.STOCK_APPLY,'_Fill','<Control>f','Automatically fill in the current square if possible.',
+             self.auto_fill_current_square_cb),
+            ('Autofill',gtk.STOCK_REFRESH,'Fill _all squares','<Control>a','Automatically fill in all squares for which there is only one valid value.',
              self.auto_fill_cb),
             ('ZoomIn',gtk.STOCK_ZOOM_IN,'_Increase size',
              '<Control>plus','Increase the size of numbers and squares',
@@ -160,6 +168,8 @@ class UI (gconf_wrapper.GConfWrapper):
             ('ZoomOut',gtk.STOCK_ZOOM_OUT,'_Decrease size',
              '<Control>minus','Decrease the size of numbers and squares.',
              self.zoom_out_cb),
+            ('FullScreen',gtk.STOCK_FULLSCREEN,None,
+             'F11',None,self.full_screen_cb),
             ('PuzzleInfo',gtk.STOCK_ABOUT,'Puzzle _Statistics',
              None,'Show statistics about current puzzle',
              self.show_info_cb),
@@ -173,11 +183,6 @@ class UI (gconf_wrapper.GConfWrapper):
              None,'Show high scores or replay old games.',
              self.show_high_scores_cb),
             ])
-        #if AUTO_SAVE:
-        #    self.main_actions.get_action('Quit').set_visible(False)
-        #    self.main_actions.get_action('Save').set_visible(False)
-        # else:
-        #    self.main_actions.get_action('Close').set_visible(False)
         self.main_actions.add_toggle_actions([
             ('AlwaysShowPossible',
              None,
@@ -232,23 +237,40 @@ class UI (gconf_wrapper.GConfWrapper):
         self.w.add_accel_group(self.uimanager.get_accel_group())
         mb = self.uimanager.get_widget('/MenuBar')
         mb.show()
-        self.vb.pack_start(mb,False)
+        self.vb.pack_start(mb,fill=False,expand=False)
         self.tb = self.uimanager.get_widget('/Toolbar')
         #self.tb.show()
-        self.vb.pack_start(self.tb,False)
+        self.vb.pack_start(self.tb,fill=False,expand=False)
         self.main_area = gtk.HBox()
         #align = gtk.Alignment()
         #align.add(self.main_area)
         #align.set_padding(18,12,0,18)
         self.vb.pack_start(self.main_area,True,padding=18)
-        self.main_area.pack_start(self.gsd,True,padding=18)
+        self.sw_for_gsd = gtk.ScrolledWindow()
+        self.sw_for_gsd.set_shadow_type(gtk.SHADOW_NONE)
+        if self.gconf['zoom_on_resize']:
+            self.sw_for_gsd.set_policy(gtk.POLICY_AUTOMATIC,
+                                       gtk.POLICY_AUTOMATIC)
+        else:
+            self.sw_for_gsd.set_policy(gtk.POLICY_NEVER,
+                                       gtk.POLICY_NEVER)
+        rect = self.sw_for_gsd.get_allocation()
+        self.cur_size = rect.width>rect.height and rect.height or rect.width
+        self.sw_for_gsd.connect('size-allocate',self.allocate_cb)
+        self.sw_vp = gtk.Viewport()
+        self.sw_vp.set_shadow_type(gtk.SHADOW_NONE)
+        self.sw_vp.add(self.gsd)
+        self.sw_for_gsd.add(self.sw_vp)
+        self.sw_vp.show(); self.sw_for_gsd.show()
+        self.main_area.pack_start(self.sw_for_gsd,expand=True,fill=True,padding=18)
+        #self.main_area.pack_start(self.gsd,expand=True,fill=True,padding=18)
         self.hint_label = gtk.Label()
         self.hint_label.show()
         align = gtk.Alignment()
         align.set_padding(18,12,12,18)
         align.add(self.hint_label)
         align.show()
-        self.vb.add(align)
+        self.vb.pack_start(align,fill=False,expand=False)
         self.main_actions.set_visible(True)
         self.game_box = gtk.VBox()
         self.main_area.show()
@@ -274,16 +296,18 @@ class UI (gconf_wrapper.GConfWrapper):
         #galign = gtk.Alignment()
         #galign.add(self.game_box)
         #galign.set_padding(18,12,12,18)
-        self.main_area.pack_start(self.game_box,True,padding=12)
+        self.main_area.pack_start(self.game_box,False,padding=12)
         #self.main_area.pack_start(galign,True,padding=12)
         # Set up trackers
         self.trackers = {}
         self.setup_tracker_interface()
         #self.game_box.pack_start(self.add_tracker_button,False)
+        #self.sw.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
+        #self.sw.add(self.vb)
+        #self.sw.show()
         self.w.add(self.vb)
         self.worker_connections=[]
         mb.show()
-
         # sync up toggles with gconf values...
         map(lambda tpl: self.gconf_wrap_toggle(*tpl),
             [('always_show_hints',
@@ -337,7 +361,7 @@ class UI (gconf_wrapper.GConfWrapper):
         else:
             # select an easy puzzle...
             puz,d=self.sudoku_tracker.get_new_puzzle(self.gconf['difficulty'])
-            print 'Default to ',puz
+            #print 'Default to ',puz
             self.gsd.change_grid(puz,
                                  9)
         # generate puzzles while our use is working...
@@ -351,10 +375,8 @@ class UI (gconf_wrapper.GConfWrapper):
         # Set up auto-resizing of font
         self.just_resized=False        
         self.resizing = False
-        rect = self.w.window.get_frame_extents()
-        self.cur_size = (rect.width,rect.height)
-        self.w.connect('size-allocate',self.allocate_cb)
 
+    @simple_debug
     def start_worker_thread (self, *args):
         n_new_puzzles = len(self.sudoku_tracker.list_new_puzzles())
         if n_new_puzzles < self.gconf['minimum_number_of_new_puzzles']:
@@ -364,18 +386,20 @@ class UI (gconf_wrapper.GConfWrapper):
                 self.timer.connect('timing-stopped',self.sudoku_maker.pause)
                 ]
             self.worker.start()
-        else:
-            print 'We already have ',n_new_puzzles,'!'
+        #else:
+        #    print 'We already have ',n_new_puzzles,'!'
 
+    @simple_debug
     def stop_worker_thread (self, *args):
         if hasattr(self,'worker'):
             self.sudoku_maker.stop()
             for c in self.worker_connections:
                 self.timer.disconnect(c)
 
+    @simple_debug
     def you_win_callback (self,grid):
         self.won = True
-        print 'you_win_callback!'
+        #print 'you_win_callback!'
         self.gconf['difficulty']=self.gconf['difficulty']+0.1
         self.timer.finish_timing()
         self.sudoku_tracker.finish_game(self)
@@ -405,24 +429,41 @@ class UI (gconf_wrapper.GConfWrapper):
         self.gsd.blank_grid()
         self.stop_game()
 
+    @simple_debug
     def allocate_cb (self, window, rectangle):
-        #print 'previous size: ',self.cur_size
-        #print 'configure event: ',rect,rect.width, rect.height
-        if self.gconf['zoom_on_resize'] and not self.just_resized:
-            rect = self.w.window.get_frame_extents()
-            #print self.cur_size,'->',rect.width,rect.height
-            if rect.width/self.cur_size[0] < rect.height/self.cur_size[1]:
-                ratio = float(rect.width) / self.cur_size[0]
-            else:
-                ratio = float(rect.height) / self.cur_size[1]
-            if ratio!=1:
-                self.cur_size = (rect.width, rect.height)
-                print 'changing font size ->',ratio
-                self.gsd.change_font_size(multiplier=ratio)
-            self.just_resized=True
-            gobject.timeout_add(200,lambda *args: setattr(self,'just_resized',False))
+        rect = window.get_allocation()
+        #print 'font-size=',self.gsd.get_font_size()
+        #print self.cur_size,'->',rect.width,rect.height,
+        short_side = (rect.width > rect.height and rect.height) or rect.width
+        if self.gconf['zoom_on_resize'] and self.cur_size != short_side:
+            #print 'changing'
+            #print 'Short side = ',short_side
+            font_size = short_side * 50
+            if font_size < self.gconf['minimum_font_size']:
+                self.gsd.change_font_size(size=self.gconf['minimum_font_size'])
+                return
+            self.gsd.change_font_size(font_size)
+            #print 'New font size = ',font_size            
+            self.cur_size = short_side
+            while (
+                (
+                (self.sw_for_gsd.get_hscrollbar() and
+                 self.sw_for_gsd.get_hscrollbar().get_property('visible')
+                 )
+                or
+                (self.sw_for_gsd.get_vscrollbar() and
+                 self.sw_for_gsd.get_vscrollbar().get_property('visible')
+                 )
+                ) 
+                and
+                font_size > self.gconf['minimum_font_size']
+                ):
+                #print 'Scrollbar visible - new font size = ',font_size
+                font_size = font_size - (0.5 * pango.SCALE)
+                self.gsd.change_font_size(size=font_size)
+                while gtk.events_pending(): gtk.main_iteration()
 
-            
+    @simple_debug
     def initialize_prefs (self):
         for k,v in self.initial_prefs.items():
             try:
@@ -430,6 +471,7 @@ class UI (gconf_wrapper.GConfWrapper):
             except:
                 self.gconf[k]=v
 
+    @simple_debug
     def new_cb (self,*args):        
         gs = game_selector.NewGameSelector(self.sudoku_tracker)
         gs.difficulty = self.gconf['difficulty']
@@ -440,6 +482,7 @@ class UI (gconf_wrapper.GConfWrapper):
             self.stop_game()
             self.gsd.change_grid(puz,9)
 
+    @simple_debug
     def stop_game (self):
         if self.gsd.grid and self.gsd.grid.is_changed():
             self.sudoku_tracker.save_game(self)
@@ -449,28 +492,33 @@ class UI (gconf_wrapper.GConfWrapper):
         self.timer.start_timing()
         self.won = False
 
+    @simple_debug
     def quit_cb (self, *args):
         #if AUTO_SAVE and not self.won: self.gconf['current_game']=self.save_game()
+        self.w.hide()
+        # make sure we really go away before doing our saving --
+        # otherwise we appear sluggish.
+        while gtk.events_pending(): gtk.main_iteration()
         if self.won:
             self.gconf['current_game']=''
         if not self.won:
             if not self.gsd.grid:
                 self.gconf['current_game']=''
             elif self.gsd.grid.is_changed():
-                print 'Saving game!'
                 self.gconf['current_game']=self.sudoku_tracker.save_game(self)
             else:
                 self.gconf['current_game']=self.gsd.grid.to_string()
-        self.w.hide()
         self.stop_worker_thread()
         self.sudoku_tracker.save()
         gtk.main_quit()
 
+    @simple_debug
     def enter_game_by_hand (self, *args):
         self.stop_game()
         self.gsd.change_grid(sudoku.InteractiveSudoku(),9)
         for w in self.by_hand_widgets: w.show_all()
         
+    @simple_debug
     def save_handmade_grid (self, *args):
         for w in self.by_hand_widgets: w.hide()
         # this should make our active grid into our virgin grid
@@ -478,9 +526,11 @@ class UI (gconf_wrapper.GConfWrapper):
         self.gsd.change_grid(self.gsd.grid,9)
         self.sudoku_maker.names[self.gsd.grid.to_string()]=self.sudoku_maker.get_puzzle_name('Custom Puzzle')
 
+    @simple_debug
     def cancel_handmade_grid (self, *args):
         for w in self.by_hand_widgets: w.hide()
 
+    @simple_debug
     def open_game (self, *args):
         #game_file = dialog_extras.getFileOpen(_("Load saved game"),
         #                        default_file=os.path.join(DATA_DIR,'games/')
@@ -494,6 +544,7 @@ class UI (gconf_wrapper.GConfWrapper):
             self.stop_game()
             saver.open_game(self,puzzl)
 
+    @simple_debug
     def save_game (self, *args):
         save_to_dir=os.path.join(DATA_DIR,'games/')
         if not os.path.exists(save_to_dir):
@@ -508,18 +559,31 @@ class UI (gconf_wrapper.GConfWrapper):
         saver.pickle_game(self, game_loc)
         return game_loc
     
+    @simple_debug
     def zoom_in_cb (self,*args):
         self.just_resized=True
         gobject.timeout_add(200,lambda *args: setattr(self,'just_resized',False))
         self.gsd.change_font_size(multiplier=1.1)
         self.zoom = self.zoom * 1.1        
 
+    @simple_debug
     def zoom_out_cb (self,*args):
         self.just_resized=True
         gobject.timeout_add(100,lambda *args: setattr(self,'just_resized',False))
         self.gsd.change_font_size(multiplier=0.9)
         self.zoom = self.zoom * 0.9
 
+    def full_screen_cb (self, *args):
+        if not hasattr(self,'is_fullscreen'): self.is_fullscreen = False
+        if self.is_fullscreen:
+            self.w.unfullscreen()
+            self.is_fullscreen = False
+        else:
+            self.w.fullscreen()
+            self.is_fullscreen = True
+        
+        
+    @simple_debug
     def clear_cb (self,*args):        
         clearer=Undo.UndoableObject(
             lambda *args: self.cleared.append(self.gsd.reset_grid()), #action
@@ -528,20 +592,23 @@ class UI (gconf_wrapper.GConfWrapper):
             )
         clearer.perform()
 
+    @simple_debug
     def show_hint_cb (self, *args):
-        print 'show_hint_cb'
         self.gsd.show_hint(self.hint_label)
 
+    @simple_debug
     def auto_hint_cb (self, action):
         if action.get_active(): self.gsd.label = self.hint_label
         else: self.gsd.label = None
 
+    @simple_debug
     def impossible_implication_cb (self, action):
         if action.get_active():
             self.gsd.show_impossible_implications = True
         else:
             self.gsd.show_impossible_implications = False
 
+    @simple_debug
     def auto_fill_cb (self, *args):
         if not hasattr(self,'autofilled'): self.autofilled=[]
         if not hasattr(self,'autofiller'):
@@ -551,16 +618,20 @@ class UI (gconf_wrapper.GConfWrapper):
                 self.history
                 )
         self.autofiller.perform()
-        print 'AUTOFILLED',self.autofilled[-1]
 
+    @simple_debug
+    def auto_fill_current_square_cb (self, *args):
+        self.gsd.auto_fill_current_entry()
+
+    @simple_debug
     def setup_tracker_interface (self):
         self.tracker_ui = TrackerBox(self)
         self.tracker_ui.show_all()
         self.tracker_ui.hide()
         self.game_box.add(self.tracker_ui)
 
+    @simple_debug
     def set_tracker (self, n):
-        #print 'Set tracker ',n
         if self.gsd.trackers.has_key(n):
             self.tracker_ui.select_tracker(n)
             e = self.gsd.get_focused_entry()
@@ -573,6 +644,7 @@ class UI (gconf_wrapper.GConfWrapper):
         else:
             print 'No tracker ',n,'yet'
         
+    @simple_debug
     def tracker_toggle_cb (self, widg):
         if widg.get_active():
             #if len(self.tracker_ui.tracker_model)<=1:
@@ -581,10 +653,12 @@ class UI (gconf_wrapper.GConfWrapper):
         else:
             self.tracker_ui.hide()
 
+    @simple_debug
     def toggle_toolbar_cb (self, widg):
         if widg.get_active(): self.tb.show()
         else: self.tb.hide()
 
+    @simple_debug
     def show_info_cb (self, *args):
         puzzle = self.gsd.grid.virgin.to_string()
         diff = self.sudoku_tracker.get_difficulty(puzzle)
@@ -610,12 +684,14 @@ class UI (gconf_wrapper.GConfWrapper):
                                    sublabel=information)
         
 
+    @simple_debug
     def toggle_generator_cb (self, toggle):
         if toggle.get_active():
             self.start_worker_thread()
         else:
             self.stop_worker_thread()
 
+    @simple_debug
     def show_high_scores_cb (self, *args):
         hs=game_selector.HighScores(self.sudoku_tracker)
         replay_game = hs.run_dialog()
@@ -623,6 +699,7 @@ class UI (gconf_wrapper.GConfWrapper):
             self.stop_game()
             self.gsd.change_grid(replay_game,9)
 
+    @simple_debug
     def show_about (self, *args):
         about = gnome.ui.About(
             APPNAME,#appname
@@ -639,40 +716,46 @@ class UI (gconf_wrapper.GConfWrapper):
         except AttributeError: pass
         about.show()
 
+    @simple_debug
     def show_faq (self, *args):
         dialog_extras.show_faq(faq_file=os.path.join(BASE_DIR,'FAQ'))
 
+    @simple_debug
     def print_game (self, *args):
         printing.print_sudokus([self.gsd])
 
+    @simple_debug
     def print_multiple_games (self, *args):
         gp=game_selector.GamePrinter(self.sudoku_tracker, self.gconf)
         gp.run_dialog()
 
+    @simple_debug
     def generate_puzzle_gui (self, *args):
         sudoku_generator_gui.GameGenerator(self,self.gconf)
 
 class TrackerBox (gtk.VBox):
 
+    @simple_debug
     def __init__ (self, main_ui):
         
         gtk.VBox.__init__(self)
         self.glade = gtk.glade.XML(os.path.join(GLADE_DIR,'tracker.glade'))
         self.main_ui = main_ui
         self.vb = self.glade.get_widget('vbox1')
-        #print 'widget ',self.vb
         self.vb.unparent()
-        self.add(self.vb)
+        self.pack_start(self.vb,expand=True,fill=True)
         self.setup_actions()
         self.setup_tree()
         self.show_all()
 
+    @simple_debug
     def reset (self):
         
         for tree in self.tracker_model:
             if tree[0]>-1:
                 self.tracker_model.remove(tree.iter)
 
+    @simple_debug
     def setup_tree (self):
         self.tracker_tree = self.glade.get_widget('treeview1')
         self.tracker_model = gtk.ListStore(int,gtk.gdk.Pixbuf,str)
@@ -685,6 +768,7 @@ class TrackerBox (gtk.VBox):
         self.tracker_model.append([-1,None,'No Tracker'])
         self.tracker_tree.get_selection().connect('changed',self.selection_changed_cb)
 
+    @simple_debug
     def setup_actions (self):
         self.tracker_actions = gtk.ActionGroup('tracker_actions')        
         self.tracker_actions.add_actions(
@@ -709,6 +793,7 @@ class TrackerBox (gtk.VBox):
         self.glade.get_widget('AddTrackerButton').connect('clicked',
                                                           self.add_tracker)
 
+    @simple_debug
     def add_tracker (self,*args):
         #print 'Adding tracker!'
         tracker_id = self.main_ui.gsd.create_tracker()
@@ -726,11 +811,13 @@ class TrackerBox (gtk.VBox):
                                   )
             )
         
+    @simple_debug
     def select_tracker (self, tracker_id):
         for row in self.tracker_model:
             if row[0]==tracker_id:
                 self.tracker_tree.get_selection().select_iter(row.iter)
 
+    @simple_debug
     def selection_changed_cb (self, selection):
         mod,itr = selection.get_selected()
         if itr: selected_tracker_id = mod.get_value(itr,0)
@@ -744,16 +831,19 @@ class TrackerBox (gtk.VBox):
                 self.main_ui.gsd.toggle_tracker(tid,tid==selected_tracker_id)
         self.tracker_actions.set_sensitive(selected_tracker_id != -1)
 
+    @simple_debug
     def clear_cb (self, action):
         mod,itr=self.tracker_tree.get_selection().get_selected()
         selected_tracker_id=mod.get_value(itr,0)
         self.tracker_delete_tracks(selected_tracker_id)
 
+    @simple_debug
     def keep_cb (self, action):
         mod,itr=self.tracker_tree.get_selection().get_selected()
         selected_tracker_id=mod.get_value(itr,0)
         self.tracker_keep_tracks(selected_tracker_id)
 
+    @simple_debug
     def tracker_delete_tracks (self, tracker_id):
         clearer=Undo.UndoableObject(
             lambda *args: self.main_ui.cleared.append(self.main_ui.gsd.delete_by_tracker(tracker_id)),
@@ -761,6 +851,7 @@ class TrackerBox (gtk.VBox):
             self.main_ui.history)
         clearer.perform()
 
+    @simple_debug
     def tracker_keep_tracks (self, tracker_id):
         clearer=Undo.UndoableObject(
             lambda *args: self.main_ui.cleared.append(self.main_ui.gsd.delete_except_for_tracker(tracker_id)),
@@ -770,9 +861,12 @@ class TrackerBox (gtk.VBox):
 
 
 class GamesTracker (sudoku_maker.SudokuTracker):
+
+    @simple_debug
     def __init__ (self, sudoku_maker):
         SudokuTracker.__init__(self, sudoku_maker)
 
+    @simple_debug
     def build_model (self):
         # puzzle / difficulty / % completed / game started / game finished
         self.model = gtk.TreeModel(str, str )
