@@ -1,6 +1,7 @@
 # GUI to Sudoku
 import gtk, sudoku, math, gobject
 import pango, random
+from simple_debug import simple_debug
 
 class ParallelDict (dict):
     """A handy new sort of dictionary for tracking conflicts.
@@ -39,8 +40,9 @@ class ParallelDict (dict):
 
 class Fonts:
     read_only_font = pango.FontDescription()
-    read_only_font.set_weight(pango.WEIGHT_BOLD)
+    read_only_font.set_weight(pango.WEIGHT_ULTRABOLD)
     read_only_font.set_size(pango.SCALE * 16)
+    read_only_color = "#666"
     normal_font = pango.FontDescription()
     normal_font.set_weight(pango.WEIGHT_NORMAL)
     normal_font.set_size(pango.SCALE * 16)
@@ -61,6 +63,7 @@ class Fonts:
               ]
 
 def change_font_size (size):
+    size = int(size)
     Fonts.normal_font.set_size(size)
     Fonts.read_only_font.set_size(size)
 
@@ -87,21 +90,25 @@ class NumberEntry (gtk.Entry):
         self.connect('focus-out-event',self.focus_out_cb)
         self.connect('key-press-event',self.keypress_cb)
         self.connect('changed',self.changed_cb)
-        self.color = None
+        self.unset_color()
         self.show()
 
+
     def changed_cb (self, *args):
         if self.__internal_change__:
             self.emit_stop_by_name('changed')
             return True
 
     def set_read_only (self, val):
-        self.set_editable(not val)
-        self.set_sensitive(not val)
+        self.set_editable(not val)        
+        #self.set_sensitive(not val)
         self.read_only = val
-        if val: self.modify_font(Fonts.read_only_font)
+        if val:
+            self.modify_font(Fonts.read_only_font)
+            if not self.color: self._set_color_(Fonts.read_only_color)
         else:
             self.modify_font(Fonts.normal_font)
+            if not self.color: self._set_color_(Fonts.normal_color)
 
     def _set_color_ (self, color):
         color = self.get_colormap().alloc_color(color)
@@ -113,14 +120,15 @@ class NumberEntry (gtk.Entry):
 
     def unset_color (self):
         self.color = None
-        self._set_color_(Fonts.normal_color)
+        if self.read_only: self._set_color_(Fonts.read_only_color)
+        else: self._set_color_(Fonts.normal_color)
 
     def set_error_highlight (self, val):
         if val:
             self._set_color_(Fonts.error_color)
-            self.set_sensitive(True)
+            #self.set_sensitive(True)
         else:
-            if self.read_only: self.set_sensitive(False)
+            #if self.read_only: self.set_sensitive(False)
             if self.color:
                 self._set_color_(self.color)
             else:
@@ -177,24 +185,24 @@ class NumberEntry (gtk.Entry):
         parent = widget.get_parent()
         while parent and not isinstance(parent,gtk.Window) :
             parent = parent.get_parent()
-        if name == 'Left':
-            #widget.emit('focus-out-event',event)
+        if name in ['Left','KP_Left']:
             parent.emit('move-focus',gtk.DIR_LEFT)
             return True
-        elif name == 'Right':
-            #widget.emit('focus-out-event',event)
+        elif name in ['Right','KP_Right']:
             parent.emit('move-focus',gtk.DIR_RIGHT)
             return True
         
         
 class EntryGrid (gtk.Table):
     def __init__ (self, group_size=9):
-        gtk.Table.__init__(self,rows=group_size,columns=group_size,homogeneous=True)
+        gtk.Table.__init__(self,rows=group_size,columns=group_size,homogeneous=True,)
+        self.set_row_spacings(4)
+        self.set_col_spacings(4)
         self.group_size = int(group_size)
         box_side = int(math.sqrt(self.group_size))
         for n in range(1,box_side):
-            self.set_row_spacing(box_side*n-1,4)
-            self.set_col_spacing(box_side*n-1,4)
+            self.set_row_spacing(box_side*n-1,12)
+            self.set_col_spacing(box_side*n-1,12)
         self.__entries__ = {}
         for x in range(self.group_size):
             for y in range(self.group_size):
@@ -202,8 +210,8 @@ class EntryGrid (gtk.Table):
                 e.x = x
                 e.y = y
                 self.attach(e,x,x+1,y,y+1,
-                            xoptions=gtk.SHRINK,
-                            yoptions=gtk.SHRINK,
+                            xoptions=gtk.FILL,
+                            yoptions=gtk.FILL,
                             xpadding=0,
                             ypadding=0)
                 self.__entries__[(x,y)]=e
@@ -213,9 +221,14 @@ class EntryGrid (gtk.Table):
         for e in self.__entries__.values():
             if e.is_focus(): return e
 
-    def change_font_size (self, size=None, multiplier=1):
-        if not size and not multiplier: raise "No size given you dumbass"
-        if multiplier:
+    def get_font_size (self, size=None):
+        size = Fonts.normal_font.get_size() or pango.SCALE * 12
+        return size
+
+    def change_font_size (self, size=None, multiplier=None):
+        if not size and not multiplier:
+            raise "No size given you dumbass"
+        elif multiplier:
             size = multiplier * (Fonts.normal_font.get_size() or pango.SCALE * 12)
         change_font_size(size)
         for e in self.__entries__.values():
@@ -228,6 +241,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
         'puzzle-finished':(gobject.SIGNAL_RUN_LAST,gobject.TYPE_NONE,())
         }
     
+    @simple_debug
     def __init__ (self,grid=None,group_size=9,
                   show_impossible_implications=False):
         group_size=int(group_size)
@@ -248,12 +262,14 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
             e.connect('focus-in-event',self.focus_callback)
             #e.connect('clicked',self.focus_callback)
 
+    @simple_debug
     def focus_callback (self, e, event):
         if hasattr(self,'hint_in_label') and self.hint_in_label: self.hint_in_label.set_text('')
         self.focused = e
         if hasattr(self,'label'):
             self.show_hint(self.label)
 
+    @simple_debug
     def show_hint (self, label):
         self.hints += 1
         self.hint_in_label = label
@@ -271,10 +287,12 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
             else:
                 label.set_text("")
 
+    @simple_debug
     def num_to_str (self, n):
         if n >= 10: return NumberEntry.conversions[n]
         else: return str(n)
 
+    @simple_debug
     def reset_grid (self):
         """Reset grid to its original setup.
 
@@ -291,6 +309,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
                         self.grid.remove(x,y)
         return removed
     
+    @simple_debug
     def blank_grid (self):
         for x in range(self.group_size):
             for y in range(self.group_size):
@@ -299,6 +318,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
                 e.set_read_only(False)                
         self.grid = None
 
+    @simple_debug
     def change_grid (self, grid, group_size):
         self.blank_grid()
         self.setup_grid(grid,group_size)
@@ -309,6 +329,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
         self.__trackers_tracking__ = {}
         self.__colors_used__ = [Fonts.error_color, Fonts.normal_color]
 
+    @simple_debug
     def load_game (self, game):
         """Load a game.
 
@@ -327,6 +348,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
                 if values[index] and not self.grid._get_(col,row):
                     self.add(col,row,values[index])
 
+    @simple_debug
     def setup_grid (self, grid, group_size):
         self.doing_initial_setup = True
         self.__error_pairs__ = ParallelDict()
@@ -340,6 +362,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
                 if val: self.add(x,y,val)
         self.doing_initial_setup = False
 
+    @simple_debug
     def entry_callback (self, widget, *args):
         if not widget.get_text():
             if self.grid and self.grid._get_(widget.x,widget.y):
@@ -350,6 +373,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
         if self.show_impossible_implications:
             self.mark_impossibile_implications(widget.x,widget.y)
 
+    @simple_debug
     def entry_validate (self, widget, *args):
         val = widget.get_value()
         try:
@@ -372,6 +396,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
                 self.__entries__[conflict].set_error_highlight(True)
             self.__error_pairs__[(err.x,err.y)]=conflicts
 
+    @simple_debug
     def add (self, x, y, val, trackers=[]):
         """Add value val at position x,y.
 
@@ -399,6 +424,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
                     self.__entries__[(x,y)].set_color(self.get_tracker_color(k))
                     self.trackers[k].append((x,y,val))
 
+    @simple_debug
     def remove (self, x, y, do_removal=False):
         """Remove x,y from our visible grid.
 
@@ -418,7 +444,7 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
                     # added to our internal grid, in which case we'd
                     # better make sure it is...
                     if not self.grid._get_(linked_entry.x,linked_entry.y):
-                        self.grid.add(linked_entry.x,linked_entry.y,linked_entry.get_value())
+                        self.add(linked_entry.x,linked_entry.y,linked_entry.get_value()) 
         # remove trackers
         for t in self.trackers_for_point(x,y):
             remove = []
@@ -432,10 +458,11 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
         if do_removal and self.grid:
             self.grid.remove(x,y)
 
+    @simple_debug
     def auto_fill (self):
-        #changed=self.grid.auto_fill()
+        changed=self.grid.auto_fill()
         #changed=self.grid.fill_must_fills        
-        changed=self.grid.fill_deterministically()
+        #changed=self.grid.fill_deterministically()
         retval = []
         for coords,val in changed:
             self.add(coords[0],coords[1],val)
@@ -444,7 +471,15 @@ class SudokuGridDisplay (EntryGrid, gobject.GObject):
         if self.grid.check_for_completeness():
                 self.emit('puzzle-finished')
         return retval
+
+    @simple_debug
+    def auto_fill_current_entry (self):
+        e = self.get_focused_entry()
+        filled = self.grid.auto_fill_for_xy(e.x,e.y)
+        if filled and filled!=-1:
+            self.add(filled[0][0],filled[0][1],filled[1])
     
+    @simple_debug
     def mark_impossibile_implications (self, x, y):
         implications = self.grid.find_impossible_implications(x,y)
         if implications:
@@ -527,9 +562,9 @@ if gtk.pygtk_version[1]<8: gobject.type_register(SudokuGridDisplay)
 
 if __name__ == '__main__':
     #eg = EntryGrid()
-    size = 4
+    size = 9
     sg = SudokuGridDisplay(
-        #grid=sudoku.fiendish_sudoku,
+        grid=sudoku.fiendish_sudoku,
         #grid=sudoku.hard_hex_sudoku,
         group_size=size,
         )
