@@ -16,7 +16,6 @@ digit_set = range(1,GROUP_SIZE+1)
 sets = [digit_set] * 9
 
 def is_set (row):
-    #print 'checking ',row
     if len(row)==len(set(row)): return True
 
 def is_sudoku (rows):
@@ -73,7 +72,7 @@ class SudokuGrid:
             self.rows.append(set())
             self.boxes.append(set())
             self.grid.append([0]*self.group_size)
-        self.grid = Numeric.array(self.grid,typecode='b')
+        self.grid = Numeric.array(self.grid,typecode=Numeric.Int)
         self.box_by_coords = {}
         self.box_coords = {}
         self.calculate_box_coords() # sets box_coords and box_by_coords
@@ -91,8 +90,6 @@ class SudokuGrid:
                 for row in range(side):
                     start = row*int(side)
                     grid.append([int(i) for i in g[start:start+side]])
-            #print 'populating from grid->',
-            #print grid
             self.populate_from_grid(grid)
         self.verbose=verbose
 
@@ -133,13 +130,9 @@ class SudokuGrid:
         self.rows[y].add(val)
         self.cols[x].add(val)
         self.boxes[box].add(val)
-        if self.verbose:
-            print 'Set ',x,',',y,'=',val            
         self._set_(x,y,val)
-        #if self.verbose: print self
 
     def remove (self, x, y):
-        #print 'Removing ',x,y
         val = self._get_(x,y)
         self.rows[y].remove(val)
         self.cols[x].remove(val)
@@ -213,6 +206,37 @@ class SudokuSolver (SudokuGrid):
         self.trail = []
         #self.complete_crumbs = BreadcrumbTrail()
 
+    def auto_fill_for_xy (self, x, y):
+        """Fill the square x,y if possible."""
+        possible = self.gen_set - self.rows[y] - self.cols[x] - self.boxes[self.box_by_coords[(x,y)]]
+        changed = []
+        if len(possible)==1:
+            val = possible.pop()
+            self.add(x,y,val)
+            return ((x,y),val)
+        if len(possible)==0:
+            return -1
+        # check our column...
+        for coord_set,filled_set in [(self.col_coords[x],self.cols[x]),
+                                     (self.row_coords[y],self.rows[y]),
+                                     (self.box_coords[self.box_by_coords[(x,y)]],
+                                      self.boxes[self.box_by_coords[(x,y)]])
+                                     ]:
+            needed_set = self.gen_set - filled_set
+            for coord in coord_set:
+                if self._get_(*coord): continue
+                elif (x,y)!=coord:
+                    needed_set = needed_set - self.possible_values(*coord)
+            if needed_set and len(needed_set)==1:
+                val = needed_set.pop()
+                if val in possible:
+                    self.add(x,y,val)
+                    return ((x,y),val)
+                else:
+                    return -1
+            if len(needed_set)>1:
+                return -1
+            
     def auto_fill (self):
         changed = []
         try: change = self.fill_must_fills()
