@@ -23,7 +23,7 @@ class SudokuGenerator:
             for y in range(self.group_size):
                 self.all_coords.append((x,y))
         if start_grid:
-            self.start_grid = sudoku.SudokuGrid(grid)
+            self.start_grid = sudoku.SudokuGrid(start_grid)
         else:
             try:
                 self.start_grid = self.generate_grid()
@@ -531,7 +531,7 @@ if __name__ == '__main__':
     import time
     for n in range(20):
         start = time.time()
-        sg = SudokuGenerator()
+        sg = SudokuGenerator(16)
         result = sg.make_unique_puzzle()
         if result: print result
         else: print 'Failed...'
